"""Preconfigured converters for the stdlib json."""
from base64 import b85decode, b85encode
from datetime import datetime

from .._compat import Counter, Set
from ..converters import Converter, GenConverter


def configure_converter(converter: Converter):
    """
    Configure the converter for use with the stdlib json module.

    * bytes are serialized as base64 strings
    * datetimes are serialized as ISO 8601
    * counters are serialized as dicts
    * sets are serialized as lists
    """
    converter.register_unstructure_hook(
        bytes, lambda v: (b85encode(v) if v else b"").decode("utf8")
    )
    converter.register_structure_hook(bytes, lambda v, _: b85decode(v))
    converter.register_unstructure_hook(datetime, lambda v: v.isoformat())
    converter.register_structure_hook(
        datetime, lambda v, _: datetime.fromisoformat(v)
    )


def make_converter(*args, **kwargs) -> GenConverter:
    kwargs["unstruct_collection_overrides"] = {
        **kwargs.get("unstruct_collection_overrides", {}),
        Set: list,
        Counter: dict,
    }
    res = GenConverter(*args, **kwargs)
    configure_converter(res)

    return res



"""Preconfigured converters for pyyaml."""
from datetime import datetime

from .._compat import FrozenSetSubscriptable
from ..converters import GenConverter
from . import validate_datetime


def configure_converter(converter: GenConverter):
    """
    Configure the converter for use with the pyyaml library.

    * frozensets are serialized as lists
    * string enums are converted into strings explicitly
    """
    converter.register_unstructure_hook(
        str, lambda v: v if v.__class__ is str else v.value
    )
    converter.register_structure_hook(datetime, validate_datetime)


def make_converter(*args, **kwargs) -> GenConverter:
    kwargs["unstruct_collection_overrides"] = {
        **kwargs.get("unstruct_collection_overrides", {}),
        FrozenSetSubscriptable: list,
    }
    res = GenConverter(*args, **kwargs)
    configure_converter(res)

    return res
